/*
	Pinguino + Mod-ZIGBEE
			   + Mod-ZIGBEE-PIR (1)
			   + Mod-ZIGBEE-PIR (2) ...
	example.
	
	Instructions will appear on the terminal.
	Also, refer to the project README for
	more information.
	
	If you have any questions, email
	support@olimex.com
	
	Olimex, May 2012
	http://www.olimex.com/dev
	
*/

#include <serial.c>
#include <__cdc.c>

#if ( defined(PIC32_PINGUINO_OTG) || defined(PIC32_PINGUINO) || defined(PIC32_PINGUINO_MICRO) )

	//Definitions for 
	// -- PIC32-PINGUINO
	// -- PIC32-PINGUINO-OTG
	// -- PIC32-PINGUINO-MICRO
	
	#define zport UART2      //The UEXT is on UART2
	#define zbaud 19200
	#define BUTTON1INIT TRISDbits.TRISD0 = 1
	
	//YELLOW LED is D1
	#define YLEDINIT   TRISDCLR = 0x02;
	#define YLED1      PORTDSET = 0x02;
	#define YLED0      PORTDCLR = 0x02;
	#define YLEDSWITCH PORTD   ^= 0x02;
	
	//GREEN LED is B15
	#define GLEDINIT  	TRISGCLR = 0x40;
	#define GLED1 		PORTGSET = 0x40;
	#define GLED0 		PORTGCLR = 0x40;
	#define GLEDSWITCH  PORTG   ^= 0x40;
	
	#define YELLOW 0
	#define GREEN  1
	#define BOTH   2
#endif



/*  Prototypes and useful functions     */


	//Blinks an LED five times - mode is YELLOW, GREEN or BOTH
	void blinkled(unsigned char mode);

	//Catches a string and writes to the buffer
	//The module sends lines in the format <string><CR><LF>
	void ZIGBEE_get_response(unsigned char uport);

	//A macro that waits for a key via CDC
	//Works with all pinguinos that support CDC
	//Syntax:
	//   WAIT;   //-> pauses the program and waits
				 //   until a key is pressed 
		
	//WAIT Macro 
	#define WAIT wait_123()
	char ch123;
	
	void wait_123() __attribute ((noinline));
	
	void wait_123()
	{
	  ch123 = CDCgetkey();
	}
	//END Wait Macro
	
	// Sends a single line of text to the terminal via CDC
	void message(char * m){ CDCprintln(m);}

	//reacts if a ZIGBEE has sent the "triggered" message
	void device_react(unsigned char dev_num);
	
/*		Global variables			  */

	#define SERIAL_BUFFER_SIZE 80
	volatile unsigned char serial_buffer[SERIAL_BUFFER_SIZE];

	#define MAX_DEVICES 5
	volatile unsigned char device_addresses[MAX_DEVICES][10];
	volatile unsigned char device_counter=0;
	volatile unsigned char device_addr_buf[10];
	
/*	    Interrupt routine definition  */

	// Interrupt routine checks if a string has been received
	// from the module and prints it on the terminal
	// Connect the wrapper to vector 4 (Timer1 overflow)
	void ISR_wrapper_vector_4(void) __attribute__ ((section (".vector_4")));

	// The wrapper calls the "Tmr1Interrupt" function
	// each time an interrupt occurs
	void ISR_wrapper_vector_4(void) { Tmr1Interrupt(); }

	//We declare the function as an interrupt routine
	void Tmr1Interrupt(void) __attribute__ ((interrupt));
	
	// configure timer 1
	void init_timer1(void)
	{
		IntConfigureSystem(INT_SYSTEM_CONFIG_MULT_VECTOR);	// interrupt mode (interrupt.c)
		T1CON=0;			// reset timer 1 configuration
		TMR1=0;				// reset timer 1 counter register
		PR1=0x9896;			// define the period (an interrupt will occur ~ 1024 times a second)
		IPC1SET=0x7;		// select interrupt priority and sub-priority
		IFS0CLR=0x10;		// clear interrupt flag
		IEC0SET=0x10;		// enable timer 1 interrupt
		T1CONCLR=0x30;	  	// set prescaler to 1:1
		T1CONSET=0x8000; 	// turn TIMER1 ON
	}
	
	
	// The interrupt routine code			
	void Tmr1Interrupt(void)
	{
		T1CONCLR = (1<<15);  //Stop Timer 1
		
		//If a string has been received, print it on the terminal
		if(SerialAvailable(zport))
		{
			ZIGBEE_get_response(zport);
			message(serial_buffer);
			
			//add a new device to the list of nodes
			if(serial_buffer[0] =='N' && serial_buffer[1]=='o')
				{
					//record the address in the addr_buf
					unsigned char i=0;
					do{
						device_addr_buf[i]= serial_buffer[i+5];
						i++;
					}while(serial_buffer[i+5] != ' ');
					device_addr_buf[i]='\0';
				
					//check if it is already present in the list of devices
					char is_new = 1;
					unsigned char j;
					for(j=0; j<MAX_DEVICES; j++)
						if( strcmp(device_addr_buf, device_addresses[j]) == 0)
								is_new = 0;
					
					if(is_new)
					{
						unsigned char i=0;
						do{
							device_addresses[device_counter][i]= device_addr_buf[i];
							i++;
						}while(device_addr_buf[i] != '\0');
						device_addresses[device_counter][i]='\0';
						
						// report on the terminal
						message("Added new device: ");
						CDCprint(device_addresses[device_counter]); CDCprint(" as device number "); CDCprintf("%d", device_counter);
						message("");
						
						if(device_counter<MAX_DEVICES-1)
							device_counter++;
					}
					
				}		
			
			// If a trigger has been detected, react
			// Note: check the function at the end of this file
			if(serial_buffer[0] =='F' && serial_buffer[1]=='r')
				{	
					
					unsigned char i=0;
					do{
						device_addr_buf[i]= serial_buffer[i+13];
						i++;
					}while(serial_buffer[i+13] != '-');
					device_addr_buf[i]='\0';
					
					unsigned char j;
					for(j=0; j<MAX_DEVICES; j++)
						if( strcmp(device_addr_buf, device_addresses[j]) == 0)
								device_react(j);
												
				}
			
			// if the network has been reset, rest the device list
			if(serial_buffer[0]=='*' && serial_buffer[1]=='*')
				{	device_counter=0;
					char k;
					for(k=0; k<MAX_DEVICES; k++)
						device_addresses[k][0]='\0';
				}
			
			
		}
		
		//turn Timer 1 on again
		init_timer1();
	}
	
	

void setup()
{	
	WAIT;
	message("Press a key to init the ZIGBEE module");
	WAIT;
	
	/* Init the Module, LEDs and Button on the pinguino board*/
		YLEDINIT;
		GLEDINIT;	
		YLED0;
		GLED0;
		BUTTON1INIT;
	
	// Open a serial communication from the board to the module
		SerialConfigure(zport, UART_ENABLE, UART_RX_TX_ENABLED, 19200);
		
	message("Connection to module is open. Now you need to:");
	message("1) Push the RST button on the MOD-ZIGBEE coordinator");
	message("---to start a fresh network. It should repost 'joining permitted' ");
	message("2) Power up/reset your ZIGBEE sensors; they should find the");
	message("---network automatically; Pinguino will report adding a new device");
	message("---at the first available position. Note: Default free positions are 5");
	message("---but you can change that in the source code of the example");
	message("3) After connecting all your devices to the network, press the");
	message("---button next to the C17 resistor on your MOD-ZIGBEE coordinator");
	
	init_timer1();
}

void loop(){}

void blinkled(unsigned char mode)
{		
			//Turn off all LEDs
			YLED0;
			GLED0;
			
			char i;
			
			// Blink
			switch(mode)
			{
				case YELLOW:
					for(i=0; i<10;i++)
					{
						YLEDSWITCH;
						delay(100);
					}
					break;
				
				case GREEN:
					for(i=0; i<10;i++)
					{
						GLEDSWITCH;
						delay(100);
					}
					break;
				
				case BOTH:
					for(i=0; i<10;i++)
					{
						YLEDSWITCH;
						GLEDSWITCH;
						delay(100);
					}
					break;
			}	
					
				
}

void ZIGBEE_get_response(unsigned char uport)
{
		char i = 0;
		char c;
		
	// Skip the <CR><LF> characters at the beginning of the string
	// (if there are any)
		c = SerialGetKey(uport);
		if(!(c== 0x0d || c== 0x0a)){serial_buffer[i] = c; i++;}
		
		c = SerialGetKey(uport);
		if(!(c== 0x0d || c== 0x0a)){serial_buffer[i] = c; i++;}
		
			
	// The rest of the code is based on the 
	// SerialGetString function in the 
	// <serial.c> library
		do {
				c = SerialGetKey(uport);
				
			    // Stop when you meet the <CR> character
				if( c != '\r')
				{	
					serial_buffer[i] = c;
					i++;
				}
						
			}while( c != '\r' );
		serial_buffer[i] = '\0';
		
	 //Catch the <LF> character
		c = SerialGetKey(uport);
		
}

// React to the trigger of each device accordingly
void device_react(unsigned char dev_num)
{
	switch(dev_num)
	{
		case 0:
			YLEDSWITCH;
			break;
		case 1:
			GLEDSWITCH
			break;
		// add more cases here if you have more devices
	}
	
	
	
	}